# `wraps-futures`

[![License][License Badge]][License]
[![Version][Version Badge]][Package]
[![Downloads][Downloads Badge]][Package]
[![Discord][Discord Badge]][Discord]

[![Documentation][Documentation Badge]][Documentation]
[![Check][Check Badge]][Actions]
[![Test][Test Badge]][Actions]
[![Coverage][Coverage Badge]][Coverage]

> *Futures feature of wraps.*

## Installing

**Python 3.8 or above is required.**

### `pip`

Installing the library with `pip` is quite simple:

```console
$ pip install wraps-futures
```

Alternatively, the library can be installed from the source:

```console
$ pip install git+https://github.com/nekitdev/wraps-futures.git
```

Or via cloning the repository:

```console
$ git clone https://github.com/nekitdev/wraps-futures.git
$ cd wraps-futures
$ pip install .
```

### `uv`

You can add `wraps-futures` as a dependency with the following command:

```console
$ uv add wraps-futures
```

## Examples

## ReAwaitable

This library implements the [`ReAwaitable[T]`][wraps.futures.core.reawaitable.ReAwaitable] type,
which wraps awaitables to allow re-awaiting.

Using the [`wrap_reawaitable`][wraps.futures.core.reawaitable.wrap_reawaitable] decorator:

```python
# reawaitable.py

from wraps.futures.core import wrap_reawaitable

@wrap_reawaitable
async def function() -> int:
    return 42
```

```python
>>> from reawaitable import function
>>> awaitable = function()
>>> await awaitable
42
>>> await awaitable
42
>>> await awaitable
42
>>> # ad infinitum...
```

## Future

[`Future[T]`][wraps.futures.core.future.Future] allows to chain asynchronous computations.

Using the [`wrap_future`][wraps.futures.core.future.wrap_future] decorator:

```python
# future.py

from wraps.futures.core import wrap_future


@wrap_future
async def function() -> int:
    return 13


async def square(value: int) -> int:
    return value * value
```

```python
>>> from future import function, square
>>> future = function().future_map_await(square).future_map(str)
>>> await future
"13"
```

[`Future[T]`][wraps.futures.core.future.Future] uses
[`ReAwaitable[T]`][wraps.futures.core.reawaitable.ReAwaitable] under the hood, so any given future
can safely be re-awaited on.

## Documentation

You can find the documentation [here][Documentation].

## Support

If you need support with the library, you can send us an [email][Email]
or refer to the official [Discord server][Discord].

## Changelog

You can find the changelog [here][Changelog].

## Security Policy

You can find the Security Policy of `wraps-futures` [here][Security].

## Contributing

If you are interested in contributing to `wraps-futures`, make sure to take a look at the
[Contributing Guide][Contributing Guide], as well as the [Code of Conduct][Code of Conduct].

## License

`wraps-futures` is licensed under the MIT License terms. See [License][License] for details.

[Email]: mailto:support@nekit.dev

[Discord]: https://nekit.dev/chat

[Actions]: https://github.com/nekitdev/wraps-futures/actions

[Changelog]: https://github.com/nekitdev/wraps-futures/blob/main/CHANGELOG.md
[Code of Conduct]: https://github.com/nekitdev/wraps-futures/blob/main/CODE_OF_CONDUCT.md
[Contributing Guide]: https://github.com/nekitdev/wraps-futures/blob/main/CONTRIBUTING.md
[Security]: https://github.com/nekitdev/wraps-futures/blob/main/SECURITY.md

[License]: https://github.com/nekitdev/wraps-futures/blob/main/LICENSE

[Package]: https://pypi.org/project/wraps-futures
[Coverage]: https://codecov.io/gh/nekitdev/wraps-futures
[Documentation]: https://nekitdev.github.io/wraps-futures

[Discord Badge]: https://img.shields.io/discord/728012506899021874
[License Badge]: https://img.shields.io/pypi/l/wraps-futures
[Version Badge]: https://img.shields.io/pypi/v/wraps-futures
[Downloads Badge]: https://img.shields.io/pypi/dm/wraps-futures

[Documentation Badge]: https://github.com/nekitdev/wraps-futures/workflows/docs/badge.svg
[Check Badge]: https://github.com/nekitdev/wraps-futures/workflows/check/badge.svg
[Test Badge]: https://github.com/nekitdev/wraps-futures/workflows/test/badge.svg
[Coverage Badge]: https://codecov.io/gh/nekitdev/wraps-futures/branch/main/graph/badge.svg

[wraps.futures.core.reawaitable.ReAwaitable]: https://nekitdev.github.io/wraps-futures/reference/core/reawaitable#wraps.futures.core.reawaitable.ReAwaitable
[wraps.futures.core.reawaitable.wrap_reawaitable]: https://nekitdev.github.io/wraps-futures/reference/core/reawaitable#wraps.futures.core.reawaitable.wrap_reawaitable

[wraps.futures.core.future.Future]: https://nekitdev.github.io/wraps-futures/reference/core/future#wraps.futures.core.future.Future
[wraps.futures.core.future.wrap_future]: https://nekitdev.github.io/wraps-futures/reference/core/future#wraps.futures.core.future.wrap_future
